@@ -697,12 +697,28 @@ int lsx_aiffstopwrite(sox_format_t * ft)
             lsx_fail_errno(ft,SOX_EOF,"non-seekable file");
             return(SOX_EOF);
         }
-        if (lsx_seeki(ft, (off_t)0, SEEK_SET) != 0)
         {
+          /* When using open_memstream(), seeking back and closing truncates
+           * the buffer to the new offset and fseek(SEEK_END) doesn't work so
+           * remember the actual length, rewrite the header and then seek back
+           * to where we were.
+           */
+          off_t o = ftell(ft->fp);
+          int result;
+
+          if (lsx_seeki(ft, (off_t)0, SEEK_SET) != 0)
+          {
                 lsx_fail_errno(ft,errno,"can't rewind output file to rewrite header");
                 return(SOX_EOF);
+          }
+          result = aiffwriteheader(ft, ft->olength / ft->signal.channels);
+          if (lsx_seeki(ft, o, SEEK_SET) != SOX_SUCCESS)
+          {
+                lsx_fail_errno(ft,errno,"can't seek back to proper place after rewriting header");
+                return(SOX_EOF);
+          }
+          return result;
         }
-        return(aiffwriteheader(ft, ft->olength / ft->signal.channels));
 }
 
 static int write_mark_and_inst_chunks(sox_format_t * ft)
@@ -904,12 +920,29 @@ int lsx_aifcstopwrite(sox_format_t * ft)
             lsx_fail_errno(ft,SOX_EOF,"non-seekable file");
             return(SOX_EOF);
         }
-        if (lsx_seeki(ft, (off_t)0, SEEK_SET) != 0)
+
+        /* When using open_memstream(), seeking back and closing truncates
+         * the buffer to the new offset and fseek(SEEK_END) doesn't work either
+         * so remember the actual length, rewrite the header and then seek back
+         * to where we were.
+         */
         {
+          off_t o = ftell(ft->fp);
+          int result;
+
+          if (lsx_seeki(ft, (off_t)0, SEEK_SET) != 0)
+          {
                 lsx_fail_errno(ft,errno,"can't rewind output file to rewrite header");
                 return(SOX_EOF);
+          }
+          result = aifcwriteheader(ft, ft->olength / ft->signal.channels);
+          if (lsx_seeki(ft, o, SEEK_SET) != SOX_SUCCESS)
+          {
+                lsx_fail_errno(ft,errno,"can't seek back to proper place after rewriting header");
+                return(SOX_EOF);
+          }
+          return result;
         }
-        return(aifcwriteheader(ft, ft->olength / ft->signal.channels));
 }
 
 static int aifcwriteheader(sox_format_t * ft, uint64_t nframes)
