@@ -472,25 +472,31 @@ static void set_endiannesses(sox_format_t * ft)
 static sox_bool is_seekable(sox_format_t const * ft)
 {
   struct stat st;
-  int fd, seekable;
+  int seekable;
+  FILE *fp;
 
   assert(ft);
   if (!ft->fp)
     return sox_false;
-  fd = fileno((FILE*)ft->fp);
-  if (fd < 0)
-     return 0;
-  fstat(fd, &st);
-  seekable = ((st.st_mode & S_IFMT) == S_IFREG);
+  fp = (FILE*)ft->fp;
+  seekable = !fseek(fp, 0, SEEK_CUR);
+
 #if defined HAVE_POSIX_FADVISE && defined POSIX_FADV_SEQUENTIAL
   if (seekable) {
-    /*
-     * POSIX_FADV_NOREUSE can potentially be beneficial, too,
-     * but is a no-op as of Linux 4.2.  Not sure about other kernels.
-     */
-    (void)posix_fadvise(fd, (off_t)0, st.st_size, POSIX_FADV_SEQUENTIAL);
+    int fd = fileno(fp);
+
+    /* open_memstream()ed and fopenmem()ed files are seekable
+     * but don't have a file descriptor */
+    if (fd >= 0 && !fstat(fd, &st)) {
+      /*
+       * POSIX_FADV_NOREUSE can potentially be beneficial, too,
+       * but is a no-op as of Linux 4.2.  Not sure about other kernels.
+       */
+      (void)posix_fadvise(fd, (off_t)0, st.st_size, POSIX_FADV_SEQUENTIAL);
+    }
   }
 #endif
+  errno = 0; /* Clear expected failures */
   return seekable;
 }
 
@@ -1257,7 +1263,11 @@ static sox_format_t * open_write(
       lsx_fail("can't set write buffer");
       goto error;
     }
-    ft->seekable = is_seekable(ft);
+    if (buffer || buffer_ptr) {
+      /* Memory buffers are always seekable */
+      ft->seekable = sox_true;
+    } else
+      ft->seekable = is_seekable(ft);
   }
 
   ft->filetype = lsx_strdup(filetype);
@@ -1372,8 +1382,11 @@ int sox_close(sox_format_t * ft)
 {
   int result = SOX_SUCCESS;
 
-  if (ft->mode == 'r')
+  if (ft->mode == 'r') {
     result = ft->handler.stopread? (*ft->handler.stopread)(ft) : SOX_SUCCESS;
+    if (ft->fp == stdin)
+      sox_globals.stdin_in_use_by = NULL;
+  }
   else {
     if (ft->handler.flags & SOX_FILE_REWIND) {
       /* Really write out a final zero byte if we're writing a sparse file.
@@ -1383,14 +1396,34 @@ int sox_close(sox_format_t * ft)
           putc('\0', (FILE *)ft->fp);
         ft->last_byte_was_zero = sox_false;
       }
+      /* If the handler wrote a header before calling the rawwrite functions,
+       * seek back to 0 and call startwrite() again to update the header.
+       *
+       * For memopen()ed files, we need to preserve the current offset
+       * otherwise it gets truncated to the new length,
+       */
+
       if (ft->olength != ft->signal.length && ft->seekable) {
+        off_t o = ftell(ft->fp);
         result = lsx_seeki(ft, (off_t)0, 0);
-        if (result == SOX_SUCCESS)
-          result = ft->handler.stopwrite? (*ft->handler.stopwrite)(ft)
-             : ft->handler.startwrite?(*ft->handler.startwrite)(ft) : SOX_SUCCESS;
+        if (result == SOX_SUCCESS) {
+          if (ft->handler.stopwrite) {
+            result = (*ft->handler.stopwrite)(ft);
+          } else if (ft->handler.startwrite) {
+            result = (*ft->handler.startwrite)(ft);
+          } else
+            result = SOX_SUCCESS;
+        }
+        fseek(ft->fp, o, SEEK_SET);
       }
     } else {
       result = ft->handler.stopwrite? (*ft->handler.stopwrite)(ft) : SOX_SUCCESS;
+
+      if (ft->fp == stdout) {
+        fflush(stdout);
+        sox_globals.stdout_in_use_by = NULL;
+      }
+
       /* Really write out a final zero byte if we're writing a sparse file.
        * See lsx_writebuf() */
       if (ft->last_byte_was_zero) {
@@ -1559,7 +1592,8 @@ static sox_bool plugins_initted = sox_false;
   #define MAX_DYNAMIC_FORMATS 42
   #define MAX_FORMATS (NSTATIC_FORMATS + MAX_DYNAMIC_FORMATS)
   #define MAX_FORMATS_1 (MAX_FORMATS + 1)
-  #define MAX_NAME_LEN (size_t)1024 /* FIXME: Use vasprintf */
+  #define MAX_NAME_LEN (size_t)32 /* To contain "lsx_%s_format_fn" */
+                                  /* FIXME: Use vasprintf */
 #else
   #define MAX_FORMATS_1
 #endif
@@ -1603,11 +1637,43 @@ sox_get_format_fns(void)
             fnname, (void *)lth, ltptr.ptr);
         if (ltptr.fn && (ltptr.fn()->sox_lib_version_code & ~255) ==
             (SOX_LIB_VERSION_CODE & ~255)) { /* compatible version check */
-          if (nformats == MAX_FORMATS) {
-            lsx_warn("too many plugin formats");
-            return -1;
+          {
+            sox_format_handler_t const * handler = ltptr.fn();
+            char const *const *namep;
+
+            /* For formats like sndfile and ffmpeg that have many
+             * format handlers, register the format-specific ones
+             * before the generic one so that, when sox_find_format() seeks
+             * a handler for a filename extension, it finds the specific
+             * one before the generic one.
+             */
+            for (namep = handler->names + 1; *namep; namep++) {
+              /* Local version so as not to overwrite the one above */
+              union {sox_format_fn_t fn; lt_ptr ptr;} ltptr;
+              char name[MAX_NAME_LEN];
+
+              sprintf(name, "lsx_%s_format_fn", *namep);
+              ltptr.ptr = lt_dlsym(lth, name);
+              if (ltptr.fn) {
+                if (nformats == MAX_FORMATS) {
+                  lsx_warn("too many plugin formats");
+                  return -1;
+                }
+                s_sox_format_fns[nformats].name = *namep;
+                s_sox_format_fns[nformats].fn = ltptr.fn;
+                nformats++;
+              }
+            }
+            /* Register the generic format handler last so that the
+             * format-specific ones take precedence over it */
+            if (nformats == MAX_FORMATS) {
+              lsx_warn("too many plugin formats");
+              return -1;
+            }
+            s_sox_format_fns[nformats].name = handler->names[0];
+            s_sox_format_fns[nformats].fn = ltptr.fn;
+            nformats++;
           }
-          s_sox_format_fns[nformats++].fn = ltptr.fn;
         }
       }
     }
